"""
@Author: obstacle
@Time: 10/01/25 14:09
@Description:  
"""
import sys

from datetime import datetime
from loguru import logger as _logger
from puti.constant.base import PuTi

_print_level = 'DEBUG'
_logfile_level = 'DEBUG'

formatted_date = datetime.now().strftime("%Y%m%d")


class LoggerFactory(object):
    _instance = None
    loggers = {}

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            obj = super(LoggerFactory, cls).__new__(cls, *args, **kwargs)
            cls._define_loggers()
            cls._instance = obj
        return cls._instance

    @classmethod
    def _define_loggers(cls, print_level=_print_level, logfile_level=_logfile_level):
        _logger.remove()
        _logger.add(sys.stderr, level=print_level, enqueue=True, backtrace=True, diagnose=True)
        _logger.level("OBSTACLES", no=38, color="<green>", icon="😋")

        # Define named loggers that write to the console stream instead of files.
        # This prevents the automatic creation of log directories.

        # Default logger
        cls.loggers['default'] = _logger.bind(name='default')

        # Client logger
        cls.loggers['client'] = _logger.bind(name='client')

        # LLM logger
        cls.loggers['llm'] = _logger.bind(name='llm')

        return cls

    @classmethod
    def _define_loggers2(cls, print_level=_print_level, logfile_level=_logfile_level):
        """
        Original file-based logging implementation, preserved for reference.
        This method is not actively used.
        """
        _logger.remove()
        _logger.add(sys.stderr, level=print_level, enqueue=True, backtrace=True, diagnose=True)
        _logger.level("OBSTACLES", no=38, color="<green>", icon="😋")

        name_default = 'default'
        _logger.add(
            str(PuTi.ROOT_DIR.val / 'logs' / name_default / f'{formatted_date}.txt'),
            filter=lambda record: record['extra'].get('name') == name_default,
            level=logfile_level,
            enqueue=True,
            backtrace=True,
            diagnose=True,
            colorize=True
        )
        cls.loggers[name_default] = _logger.bind(name=name_default)

        name_client = 'client'
        _logger.add(
            str(PuTi.ROOT_DIR.val / 'logs' / name_client / f'{formatted_date}.txt'),
            filter=lambda record: record['extra'].get('name') == name_client,
            level=logfile_level,
            enqueue=True,
            backtrace=True,
            diagnose=True,
            colorize=True
        )
        cls.loggers[name_client] = _logger.bind(name=name_client)

        name_client = 'llm'
        _logger.add(
            str(PuTi.ROOT_DIR.val / 'logs' / name_client / f'{formatted_date}.txt'),
            filter=lambda record: record['extra'].get('name') == name_client,
            level=logfile_level,
            enqueue=True,
            backtrace=True,
            diagnose=True,
            colorize=True
        )
        cls.loggers[name_client] = _logger.bind(name=name_client)

        return cls

    @property
    def default(self):
        return self.loggers['default']

    @property
    def client(self):
        return self.loggers['client']

    @property
    def llm(self):
        return self.loggers['llm']


logger_factory = LoggerFactory()



"""
@Author: obstacle
@Time: 16/01/25 11:30
@Description:  
"""
import asyncio
import json
import traceback

from twikit.errors import Unauthorized
from httpx import ConnectTimeout, ConnectError
from inspect import iscoroutinefunction
from typing import Union, Dict
from pydantic import BaseModel, ConfigDict
from puti.logs import logger_factory
from puti.utils.common import get_structured_exception, is_valid_json
from puti.core.resp import Response
from puti.constant.base import Resp


lgr = logger_factory.default


class Capture(BaseModel):
    """ exception capture """
    model_config = ConfigDict(arbitrary_types_allowed=True)

    exe: Exception = None
    desc: str = None
    max_retries: int = 1

    def invoke(self, func, *args, **kwargs) -> Response:
        if 'retries' in kwargs:
            retries = kwargs.pop('retries')
        else:
            retries = 0

        try:
            if iscoroutinefunction(func):
                loop = asyncio.get_event_loop()
                try:
                    if loop.is_running():
                        rs = loop.run_until_complete(func(*args, **kwargs))
                    else:
                        rs = asyncio.run(func(*args, **kwargs))
                except RuntimeError:
                    rs = loop.run_until_complete(func(*args, **kwargs))
            else:
                rs = func(*args, **kwargs)
        except Unauthorized as e:
            lgr.error(f'Unauthorized Error: {e}, check your credentials and account')
            structured_e = self._e_handled(e)
            return Response.default(code=Resp.UNAUTHORIZED_ERR.val, msg=Resp.UNAUTHORIZED_ERR.dsp, data=structured_e)
        except (ConnectError, ConnectTimeout) as e:
            lgr.log('OBSTACLES', 'Connect Timeout, come on, try again!')
            if retries < self.max_retries:
                return self.invoke(func, *args, retries=retries + 1, **kwargs)
            else:
                structured_e = self._e_handled(e)
                return Response.default(code=Resp.CP_ERR.val, msg=Resp.CP_ERR.dsp, data=structured_e)
        except ZeroDivisionError as e:
            structured_e = self._e_handled(e)
            return Response.default(code=Resp.CP_ERR.val, msg=Resp.CP_ERR.dsp, data=structured_e)
        except Exception as e:
            structured_e = self._e_handled(e)
            return Response.default(code=Resp.CP_ERR.val, msg=Resp.CP_ERR.dsp, data=structured_e)
        else:
            data = json.loads(rs) if is_valid_json(rs) else rs
            return Response.default(data=data)

    def _e_handled(self, e: Exception) -> Union[str, Dict[str, str]]:
        desc = traceback.format_exc()
        structured_e = get_structured_exception(e)
        self.exe = e
        self.desc = f'{structured_e}\n {desc}'
        lgr.error(f'{structured_e}\n {desc}')
        return structured_e
